/*$Id: io_error.h,v 26.52 2007/11/02 21:15:21 al Exp $ -*- C++ -*-
 * data for error and exception handling
 *
 * Copyright (C) 2001 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
//testing=trivial 2006.07.17
#ifndef bERROR
#include "md.h"
#include "io_trace.h"
/*--------------------------------------------------------------------------*/
/* arg to error() (badness) to decide severity of exception */
#define	bNOERROR	0
#define bTRACE		1
#define bLOG		2
#define bDEBUG		3
#define bPICKY		4
#define	bWARNING	5
#define bDANGER		6
#define bERROR		7
#define bEXIT		8
#define bDISASTER	9
	void	  error(int,const char*,...);
	void	  error(int,const std::string&);
/*--------------------------------------------------------------------------*/
extern std::string to_string(int);

struct Exception {
  std::string _message;
  RUN_MODE _run_mode;
  const std::string& message()const {return _message;}
  RUN_MODE run_mode()const {return _run_mode;}
  Exception(const std::string& message)
    :_message(message), _run_mode(ENV::run_mode) {
  }
};
struct Exception_No_Match :public Exception{
  std::string _key;
  Exception_No_Match(const std::string& key)
    :Exception("no match: " + key),
     _key(key) {
  }
};
struct Exception_Too_Many :public Exception{
  int _requested, _max;
  Exception_Too_Many(int requested, int max)
    :Exception("too many: requested=" + to_string(requested) + " max=" + to_string(max)),
     _requested(requested), _max(max) {untested();
  }
};
struct Exception_Cant_Set_Type :public Exception{
  std::string _from, _to;
  Exception_Cant_Set_Type(const std::string& from, const std::string& to)
    :Exception("can't set type from \"" + from + "\" to \"" + to + "\""),
     _from(from), _to(to) {untested();
  }
};
struct Exception_Bad_Internal_Command :public Exception{
  std::string _c;
  Exception_Bad_Internal_Command(const std::string& command)
    :Exception("bad internal command: " + command),
     _c(command) {untested();
  }    
};
#if 0
struct Exception_Model_Type_Mismatch :public Exception{
  std::string _device, _need_type, _got_type;
  Exception_Model_Type_Mismatch(const std::string& d, const std::string& n, const std::string& g)
    :Exception(""),
     _device(d), _need_type(n), _got_type(g) {untested();
  }
};
#endif
struct Exception_End_Of_Input :public Exception{
  Exception_End_Of_Input(const std::string& message) 
    :Exception(message) {
  }
};

struct Exception_Too_Few :public Exception{
  Exception_Too_Few(const std::string& s)
    :Exception(s) {
  }
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
