/*$Id: lang_spectre_in.cc,v 26.52 2007/11/02 21:15:21 al Exp $ -*- C++ -*-
 * Copyright (C) 2007 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include "declare.h"
#include "d_dot.h"
#include "d_coment.h"
#include "d_subckt.h"
#include "e_model.h"
#include "lang_spectre.h"
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
static void parse_type(CS& cmd, CARD* x)
{
  assert(x);
  std::string new_type;
  cmd >> new_type;
  x->set_dev_type(new_type);
}
/*--------------------------------------------------------------------------*/
static void parse_args(CS& cmd, CARD* x)
{
  assert(x);
  
  int here = 0;
  while (cmd.more() && !cmd.stuck(&here)) {
    try{
      std::string name, value;
      cmd >> name >> '=' >> value;
      x->set_param_by_name(name, value);
    }catch (Exception_No_Match) {untested();
      cmd.warn(bDANGER, here, "bad parameter, ignored");
    }
  }
}
/*--------------------------------------------------------------------------*/
static void parse_label(CS& cmd, CARD* x)
{
  assert(x);
  std::string my_name;
  cmd >> my_name;
  x->set_label(my_name);
}
/*--------------------------------------------------------------------------*/
static void parse_ports(CS& cmd, CARD* x)
{
  assert(x);

  if (cmd >> '(') {
    int index = 0;
    while (cmd.is_alnum()) {
      int here = cmd.cursor();
      try{
	std::string value;
	cmd >> value;
	x->set_port_by_index(index++, value);
      }catch (Exception_Too_Many e) {untested();
	cmd.warn(bDANGER, here, e.message());
      }
    }
    cmd >> ')';
  }else{untested();
    cmd.warn(bDANGER, "'(' required");
  }
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
void LANG_SPECTRE::parse_paramset(CS& cmd, MODEL_CARD* x)
{
  assert(x);
  cmd.reset();
  cmd >> "model";
  parse_label(cmd, x);
  parse_type(cmd, x);
  parse_args(cmd, x);
  cmd.check(bWARNING, "what's this?");
}
/*--------------------------------------------------------------------------*/
void LANG_SPECTRE::parse_module(CS& cmd, MODEL_SUBCKT* x)
{
  assert(x);
  cmd.reset();
  cmd >> "subckt";
  parse_label(cmd, x);
  parse_ports(cmd, x);

  for (;;) {
    cmd.get_line("spectre-subckt>");

    if (cmd.dmatch("ends")) {
      break;
    }else if (cmd.dmatch("parameters")) {
      cmd.reset();
      CMD::cmdproc(cmd,x->subckt());
    }else{
      cmd.dscan("(").dscan(")"); // the type follows parens, skip ahead
      new__instance(cmd, x);
    }
  }
}
/*--------------------------------------------------------------------------*/
void LANG_SPECTRE::parse_instance(CS& cmd, COMPONENT* x)
{
  cmd.reset();
  parse_label(cmd, x);
  parse_ports(cmd, x);
  parse_type(cmd, x);
  parse_args(cmd, x);
  cmd.check(bWARNING, "what's this?");
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
